import netgen.gui
from ngsolve import *
import netgen.geom2d as geom2d

geo = geom2d.SplineGeometry()
geo.AddRectangle((-1,-1), (1,1), bcs=["bottom", "right", "top", "left"])
mesh = Mesh(geo.GenerateMesh(maxh=0.05))
mesh.GetBoundaries()

fes = H1(mesh, order=2, dirichlet="left|right|top|bottom", autoupdate=True)

g = 2*exp(-1.5*(x**2 + y**2))

gfu = GridFunction(fes, autoupdate=True)
gfu.Set(g, definedon=mesh.Boundaries("left|right|top|bottom"))
Draw(gfu)

u, v = fes.TnT()
a = BilinearForm(fes)
a += grad(u)*grad(v)*dx
f = LinearForm(fes)
f += (-(18*x*x-6)*exp(-1.5*(x*x + y*y))-(18*y*y-6)*exp(-1.5*(x*x + y*y)))*v*dx
c = Preconditioner(a,"multigrid")

space_flux = HDiv(mesh, order=2, autoupdate=True)
gf_flux = GridFunction(space_flux, "flux", autoupdate=True)

def solveStep():
    gfu.Set(g, definedon=mesh.Boundaries("left|right|top|bottom"))
    Draw(gfu)
    a.Assemble()
    f.Assemble()
    solvers.BVP(bf=a, lf=f, gf=gfu, pre=c)
    Redraw(blocking=True)
    return

def estimateError():
    flux = grad(gfu)
    gf_flux.Set(flux) 
    
    err = (flux-gf_flux)*(flux-gf_flux)
    Draw(err, mesh, 'error_representation')
    eta2 = Integrate(err, mesh, VOL, element_wise=True)
    
    maxerr = max(eta2)
    for el in mesh.Elements():
        mesh.SetRefinementFlag(el, eta2[el.nr] > 0.25*maxerr)

    return

solveStep()
estimateError()
mesh.Refine()

solveStep()
